;BONES.ASM
; load bones into each players stash

;boneTable
; 8*4*3 - 5
; there are 91 bones
; each bone has a bit in the bone table
;  IF (1) it is set then 
;   the bone exists in the pile,
;  IF (0)it is zero then
;   a player has pulled the bone from the pile

; boneTable is 12 bytes long

; init routine
fillBoneTable:
 ld de,boneTable+1
 ld hl,boneTable
 ld (hl),255
 ld bc,12
 ldir

 ld a,91
 ld (piecesLeft),a

 ; NEED TO REMOVE THE CENTERPIECE
 ld a,(centerpiece)
 ld c,0
 call searchBoneTable
 inc hl
 call searchBoneTable
 ld a,c
 sla a
 push af
  and 111b
  ld b,a
 pop af
 sra a
 sra a
 sra a

 ld e,a
 ld d,0
 ld hl,boneTable
 add hl,de

 ; bone chosen
 ;  reset the byte
 call setMaskB
 ld a,(hl)
 and b
 ld (hl),a
 ret

searchBoneTable:
 ld b,(hl)
 cp a
 ret z
 inc hl
 inc hl
 inc c
 jr searchBoneTable


dispCurrentPlayerBones:
 ld hl,(curPlayer)

;in: hl
dispPlayerBones:
 ld a,(scroll)
 sla a
 ld e,a
 ld d,0
 add hl,de	; add offset

 ld c,0

dispPlayerBonesLoop:
 ld a,(hl)
 push bc

 ld e,c
 sla e
 sla e
 sla e
 ld d,0
 bit 7,a
 jr z,dontShiftPlayerBone

 inc d
 inc d
 inc d
 inc d
dontShiftPlayerBone:
 push de
 push hl
 and 1111b
 call drawDomino
 pop hl
 pop de
 ld a,d
 add a,8
 ld d,a
 inc hl
 ld a,(hl)
 push hl
  call drawDomino
 pop hl
 inc hl
 pop bc

 ; check to see if we saw the last domino
 dec hl
 dec hl
 ld a,(hl)
 bit 4,a
 ret nz
 inc hl
 inc hl

 inc c
 ld a,c
 cp 8
 jr c,dispPlayerBonesLoop
 ret


lastBoneInA:
 ld de,bones1
 sla a
 sla a
 sla a
 sla a
 sla a
 sla a
 ld l,a
 ld h,0
 add hl,hl
 add hl,de

; in: hl = pointer to bone collection
gotoLastBone:
 ld a,(hl)
 bit 4,a
 ret nz
 inc hl
 inc hl
 jr gotoLastBone

; counts
gotoLastBoneC:
 ld b,0
gotoLastBoneC0:
 ld a,(hl)
 bit 4,a
 ret nz
 inc hl
 inc hl
 inc b
 jr gotoLastBoneC0

getCurrentPiecesLeft:
 ld hl,(curPlayer)
getPiecesLeft:
 ld b,0
getPiecesLeftLoop:
 ld a,(hl)
 bit 4,a
 ret nz
 inc b
 inc hl
 inc hl
 jr getPiecesLeftLoop

; show the arrows if we need to
showScrollArrows:
 ld a,(counter)
 inc a
 and 11111b
 ld (counter),a

 ld a,(maxScroll)
 or a
 ret z

 ld b,a
 ld a,(scroll)
 cp b
 jr z,scrollUpArrows

scrollDownArrows:
 ld a,16
 ld e,53
 ld hl,scrollDownSprite
 call sprite8xor
 ld a,16
 ld e,55
 ld hl,scrollDownSprite
 call sprite8xor

 ld a,(counter)
 sra a
 sra a
 sra a
 ld e,52
 add a,e
 ld e,a
 ld a,16
 ld hl,scrollDownSprite
 jp sprite8xor
 

scrollUpArrows:
 or a
 ret z

 ld a,16
 ld e,1
 ld hl,scrollUpSprite
 call sprite8xor
 ld a,16
 ld e,3
 ld hl,scrollUpSprite
 call sprite8xor

 ld a,(counter)
 sra a
 sra a
 sra a
 ld e,a
 ld a,16
 ld hl,scrollUpSprite
 jp sprite8xor
 ret

;----------------
; FILL WITH BONES
;----------------
;in: hl=pointer to start of players bone collection
;    b=number of bones to give him
;out: bone collection updated
;----------------------------
fillWithBones:
 push bc
 push hl
  call getBone
 pop hl
  ld (hl),a
  inc hl
  ld (hl),b
  inc hl
 pop bc
 djnz fillWithBones

 dec hl
 dec hl
 ld a,(hl)
 or 00010000b
 ld (hl),a
 ret

;-----------------
; GET CURRENT BONE
;-----------------
; in: nothing
; out: hl = pointer to bone, c = piece #
; destroys: HL, BC, A
; summary:
;  hl = (curPlayer) + (bonecursor + scroll)*2
;--------------------------------------
getCurrentBone:
 ld hl,(curPlayer)	; get players stack
 ld a,(bonecursor)
 ld b,a
 ld a,(scroll)
 add a,b
 ld c,a					; # place of piece in bone stack
 sla a
 ld e,a
 ld d,0
 add hl,de		; add piece offset
 ret

; out: a=side 1, b=side 2
; preserves: de,hl,c
getCurrentPiece:
 push de
 push hl
 push bc
  call getCurrentBone
 pop bc
 ld a,(hl)
 inc hl
 ld b,(hl)
 pop hl
 pop de
 ret

;----------------
; REMOVE BONE
;----------------
; in:
 ; bonecursor = bone # in stack
 ; curPlayer
; description:
;  lose curent bone, resort bone array
;----------------
removeBone:
 call getCurrentBone

 ld a,(hl)
 bit 4,a
 jr nz,removeLastBone

 ld a,30	; maximum # of bones
 sub c
 sla a
 ld c,a
 ld b,0

 push hl
 pop de
 inc hl
 inc hl
 ldir
 ld (hl),0
 inc hl
 ld (hl),0
 ret

removeLastBone:
 dec hl
 dec hl
 ld a,(hl)
 or 00010000b
 ld (hl),a
 inc hl
 inc hl
 ld (hl),0
 inc hl
 ld (hl),0
 ret

; DEBUGGING ROUTINE
showBoneTable:
 ld hl,62
 ld (pencol),hl
 ld hl,boneTable
 ld b,11
showBoneTableLoop:
 ld a,(hl)
 
 inc hl
 push hl
  push bc
   call showBits
  pop bc
  ld a,(penrow)
  add a,6
  ld (penrow),a
  ld a,62
  ld (pencol),a
 pop hl
 djnz showBoneTableLoop
 push hl
 ld hl,50*256
 ld (pencol),hl
 pop hl
 ld a,(hl)
 call showBits
  push hl
 ld hl,56*256
 ld (pencol),hl
 pop hl
 inc hl
 ld a,(hl)
 jp showBits

; get bone debug

getBonePOPdebug:
 B_CALL GrBufClr

  ld hl,18*256
  ld (pencol),hl
 ld hl,track1
 call valueAtHL
 ld a,(track1)
 call showBits

 ld hl,30*256
 ld (pencol),hl
 ld hl,track2
 call valueAtHL
 inc hl
 call valueAtHL

 B_CALL grbufcpy
 B_CALL getkey


getBonePOP:
  pop hl

;----------
; GET BONE
;----------
;out: a=side1 of domino, b=side2 of domino
;-----------------------------------------
getBone:
 call getrand91
 ld (bonecursor),a

 ; DIVIDE BY 8
 ; b is position in bit
 ; a bytes to shift
 push af
  and 111b
  ld b,a
 pop af
 sra a
 sra a
 sra a

 ; add offset
 ld e,a
 ld d,0
 ld hl,boneTable
 add hl,de

 ; is bit set?
 push hl
  ld a,(hl)
  call checkBitB
  jr z,getBonePOP		; already taken ... get another
 pop hl

 ; bone chosen
 ;  reset the byte
 call setMaskB
 ld a,(hl)
 and b
 ld (hl),a

 ld a,(piecesLeft)
 dec a
 ld (piecesLeft),a

 ; determine what domino to give
 ld hl,boneData
 ld a,(bonecursor)
 sla a
 ld e,a
 ld d,0
 add hl,de
 ld a,(hl)
 inc hl
 ld b,(hl)
 ;A,B = sides of domino
 ret

setMaskB:
 ld a,b
 or a
 jr z,setMask0

 ld a,10000000b
setMaskLoop:
 rr a
 djnz setMaskLoop

 xor 255
 ld b,a
 ret

setMask0:
 ld b,01111111b
 ret

coverUpBones:

 ; blacken bones
 ld hl,plotsscreen
 ld de,11
 ld b,56
coverBonesLoop:
 ld (hl),255
 inc hl
 ld (hl),255
 add hl,de
 djnz coverBonesLoop

 ; cover image

 ; 12*56 = 672
 ld de,plotsscreen+672
 ld hl,coverImage
 ld b,8
coverImageLoop:
 push bc
  ld bc,3
  ldir
 pop bc
 push hl
  ld hl,9
  add hl,de
  ex de,hl
 pop hl
 djnz coverImageLoop

 ret

