;SELECTOR.ASM
; keypress reader
;
; controlLoop - select a piece
; placementLoop - select a place to play it

selectPiece:
 xor a
 ld (bonecursor),a
 ld (scroll),a

 ld a,(cover)
 bit 0,a
 jr z,dontSetCoverByte

 or 10000000b
 ld (cover),a

dontSetCoverByte:
 ld hl,(curPlayer)
 call gotoLastBoneC
 ld a,b
 cp 8
 jr c,setMaxBone

 ld a,7
setMaxBone:
 ld (maxBone),a

controlLoop:
 B_CALL grbufclr
 call showBottomRow
 call showScrollArrows	; <bones.asm>
 call showTracks
 call dispCurrentPlayerBones
 call showBonesCursor

 ; cover it up?
 ld a,(cover)
 bit 7,a
 call nz,coverUpBones	; <bones.asm>

 B_CALL GrBufCpy
controlLoop2:
 ei
 halt 

 B_CALL getkey
 cp kEnter
 jr z,selectorEnter
 cp kUp
 jr z,selectorUp
 cp kDown
 jr z,selectorDown
 cp kDel
 jr z,selectorDelete
 cp kGraph
 jr z,selectorGraph
 cp kYequ
 jr z,selectorYequals
 cp kAdd
 jr z,selectorAdd
 cp kSub
 jp z,selectorSubtract
 cp kMatrix
 jp z,selectorMatrix
 cp kMode
 jr nz,controlLoop2
 jp quit

selectorUp:
 ld a,(bonecursor)
 or a
 jr z,selectorScrollUp
 dec a
 ld (bonecursor),a
 jr controlLoop

selectorScrollUp:
 ld a,(scroll)
 or a
 jr z,controlLoop
 dec a
 ld (scroll),a
 jr controlLoop

selectorDown:
 ld a,(maxBone)
 ld b,a
 ld a,(bonecursor)
 cp b
 jr z,selectorScrollDown
 inc a
 ld (bonecursor),a
 jr controlLoop

selectorScrollDown:
 ld a,(maxScroll)
 ld b,a
 ld a,(scroll)
 cp b
 jr z,controlLoop
 inc a
 ld (scroll),a
 jr controlLoop

selectorYequals
 ld a,(cover)
 xor 10000000b
 ld (cover),a
 jr controlLoop

;------
; DELETE
;--------
; undo last move
;----------------
selectorDelete:
 ld a,(undo)
 bit 7,a
 jr nz,controlLoop

 and 111b
 cp 5
 jr z,undoDrawnPiece

 jp controlLoop

undoDrawnPiece:
 ; get piece
 ld a,(undo1)
 call lastBoneInA

 ld (hl),0
 inc hl
 ld (hl),0
 dec hl
 dec hl
 dec hl
 ld a,(hl)
 or 10000b
 ld (hl),a
 jp controlLoop

;-------
; GRAPH
;-------
; draw a new piece
;-----------------
selectorGraph:
 ; are there any untaken pieces remaining
 ld a,(piecesLeft)
 or a
 jr nz,somePiecesLeft

 ld hl,gMessage0
 call showMessage
 jp metaLoop

gMessage0: db "No Pieces Left",0

gMessage1: db "Too Many Bones",0

somePiecesLeft:
 ld hl,(curPlayer)
 call gotoLastBoneC
 ld a,b
 cp 30
 jr c,graphBonesFree


 ld hl,20
 ld (pencol),hl
 call valueForA
 ld hl,gMessage1
 call showMessage
 jp controlLoop

graphBonesFree:
 ld a,(hl)
 and 1111b
 ld (hl),a
 inc hl
 inc hl
 push hl
  call getBone
 pop hl
  or 00010000b
  ld (hl),a
  inc hl
  ld (hl),b

 ; delay then goto next player
 ei
 call delay

 ld a,(player)
 inc hl
 ld (hl),a

 call getTrackA
 call endOfTrack
 or 00100000b
 ld (hl),a
 jp metaLoop

selectorAdd:
 ld a,(vm)
 xor 1b
 ld (vm),a
 jp controlLoop

selectorSubtract:
 B_CALL grbufclr
 call showBoneTable
 B_CALL grbufcpy
 B_CALL getkey
 jp controlLoop

selectorMatrix:
 B_CALL grbufclr



 B_CALL grbufcpy
 B_CALL getkey
 jp controlLoop

;-------
; ENTER
;-------
; select a piece
;---------------
selectorEnter:
 call getCurrentBone	; <bones.asm>
 ld a,(hl)
 or 10000000b	; piece juts out
 ld (hl),a

 xor a
 ld (cursor),a
 ld (imCounter),a

;---------------------------------------------------------------------
;----------------
; PLACEMENT LOOP
;----------------
placementLoop:
 B_CALL grbufclr
 call showBottomRow
 call showTracks
 call dispCurrentPlayerBones
 call showPlacementCursor
 B_CALL GrBufCpy
placementLoop2:
 ei
 halt 

 B_CALL getkey
 cp kEnter
 jr z,placeEnter
 cp kLeft
 jr z,placeLeft
 cp kRight
 jr z,placeRight
 cp kDel
 jr z,placeFlipPiece
 cp kMode
 jr nz,placementLoop2

 ;------
 ; MODE 
 ;------
 ; return to previous menu (bone selection)
 ;-----------------------------------------
 
 ; unshift the selected bone
 call getCurrentBone
 ld a,(hl)
 and 1111b
 ld (hl),a

 jp controlLoop


;-------
; ENTER
;-------
; try to place a piece on the table
;----------------------------------
placeEnter:
 ; do pieces match (if NO then return to loop)
 call checkPlaceNOW ; <moves.asm>
 jr nc,placementLoop

 ld a,(double)
 bit 7,a
 jr nz,doubleInactive0

 and 1111b
 cp 13
 jr nc,doubleInactive1

 ld a,(double)
 sra a
 sra a
 sra a
 sra a
 and 11b
 ld b,a
 ld a,(cursor)
 cp b
 jr nz,notAtopDouble

 ; remove the double -- its played atop!

 ld a,1111b
 ld (double),a
 jr notPlayingDoubles

notAtopDouble:
 ld hl,doubleMessage1
 call showMessage
 jr placementLoop 

doubleInactive0:
 ; you are playing the next piece after a double
 ld a,(double)
 sra a
 sra a
 sra a
 sra a
 and 11b
 ld b,a
 ld a,(cursor)
 cp b
 jr nz,doubleInactive1

 ; played on your own double
 ld a,1111b
 ld (double),a
 jr notPlayingDoubles

doubleInactive1:
 ld a,255
 ld (imCounter),a

 ; doubles ?
 call getCurrentPiece
 and 1111b
 cp b
 jr nz,notPlayingDoubles

 ld (imCounter),a

 ; draw all the doubles around the message box

 call playDoubleAnimation
 ld hl,doubleMessage0
 call showMessage

notPlayingDoubles:
 ; get the last piece in the track the cursor is over
 ld a,(cursor)
 call endOfTrackA

 ld c,00110000b	; train still on
 ld a,(cursor)
 ld b,a

 ld a,(player)
 cp b
 jr nz,placeOnAnotherTrain

 ; remove train if you place on your track
 ld c,00010000b

 ; train stays on if it belongs to another player
placeOnAnotherTrain:
 ; add current piece to current track
 push hl
  ld e,00001111b
  ld a,(cursor)
  bit 0,a
  jr z,placeOnAnotherNOW

  ld e,10001111b
placeOnAnotherNOW:
  call getCurrentPiece
  and e
  ld d,a
  ld e,b
  and 10001111b
 pop hl
 call addToTrack

 ; remove current piece from players bones
 call removeBone

 ld a,(imCounter)
 cp 14
 jp nc,metaLoop

 ; double
 ;  db c?aabbbb
 ; bbbb = piece #
 ; aa = cursor #
 ; c = wait for another player...
 ld b,a
 ld a,(cursor)
 sla a
 sla a
 sla a
 sla a
 or b
 or 10000000b
 ld (double),a
 jp selectPiece

;------------
; place LEFT
;------------
placeLeft:
 ld a,(cursor)
 cp 2
 call z,flipPiece
 or a
 jr nz,placeLeft2

 call flipPiece
 ld a,4
placeLeft2:
 dec a
 ld (cursor),a
 jr placementLoop

;-------------
; place RIGHT
;-------------
placeRight
 ld a,(cursor)
 cp 1
 call z,flipPiece
 cp 3
 jr nz,placeRight2

 call flipPiece
 ld a,255
placeRight2:
 inc a
 ld (cursor),a
 jr placementLoop

;------------
; place FLIP
;------------
placeFlipPiece:
 call flipPiece
 jr placementLoop

;-----------------------
; SHOW PLACEMENT CURSOR
;-----------------------
showPlacementCursor:
 call getCurrentBone
 ld a,(cursor)
 call vm0_location

 ld c,11111111b
 ld a,(cursor)
 bit 0,a
 jr nz,drawCursorNOW

 ld c,01111111b
drawCursorNOW:
 ld a,(hl)
 and c
 inc hl
 call setTempTrack
 jp drawPiece

setTempTrack:
 push hl
  ld b,(hl)
  ld hl,tempTrack
  ld (hl),a
  inc hl
  ld (hl),b
  inc hl
  ld (hl),d
  inc hl
  ld (hl),e
 pop hl
 ret

flipPiece:
 push af
 call getCurrentBone
 ld a,(hl)
 and 11110000b
 ld c,a
 ld a,(hl)
 inc hl
 ld b,(hl)
 and 1111b
 ld (hl),a
 dec hl
 ld a,b
 or c
 ld (hl),a
 pop af
 ret

;-------------------
; SHOW BONES CURSOR
;-------------------
showBonesCursor:
 ld a,(bonecursor)
 sla a
 sla a
 sla a

 sla a
 sla a
 ld e,a
 ld d,0
 ld hl,plotsscreen
 add hl,de
 add hl,de
 add hl,de

 ld b,8
invertCursor:
 ld a,(hl)
 xor 255
 ld (hl),a
 inc hl
 ld a,(hl)
 xor 255
 ld (hl),a
 ld de,11
 add hl,de
 djnz invertCursor
 ret

doubleMessage0: db "DOUBLE",0
doubleMessage1: db "Not on double!",0
