;TRACK.ASM
; manages the train tracks, using methods similar to those of master conductors
; shows tracks

loadTracks:
 ld de,track1
 ld hl,trackdat
 ld bc,13
 ldir

 ld de,track2
 ld hl,trackdat
 ld bc,13
 ldir

 ld de,track3
 ld hl,trackdat
 ld bc,13
 ldir

 ld de,track4
 ld hl,track4dat
 ld bc,13
 ldir

 ld de,centerpiece+2
 ld hl,centerpiecedat+2
 ld bc,3
 ldir

 ;ld hl,tempTrack
 ;ld (hl),11010000b
 ret

;-------------
; ADD TO TRACK
;-------------
;in: hl = start of track
; a/b = sides of domino
; d/e = x,y
; c = end of track indicator
;result:
; add entry to the track
;-----------------------
addToTrack:
 ld (hl),a
 inc hl
 ld (hl),b
 inc hl
 ld (hl),d
 inc hl
 ld (hl),e
 inc hl
 ld (hl),c
 ret

endOfPlayerTrack:
 ld a,(player)
endOfTrackA:
 sla a
 sla a
 sla a
 sla a
 sla a
 sla a ; a*64
 ld e,a
 ld d,0
 ld hl,track1
 add hl,de
 add hl,de

;-------------
; END OF TRACK
;-------------
; in: hl - start of track
; out: hl - end of track, a = (hl)
;---------------------------------
 ld b,0
endOfTrack:
 ld a,(hl)
 bit 4,a
 ret nz

 inc hl
 inc hl
 inc hl
 inc hl
 inc b
 jr endOfTrack


; track a = track 1,2,3, or 4
getTrackA:
 sla a
 sla a
 sla a
 sla a
 sla a		; x32
 ld l,a
 ld h,0
 add hl,hl
 add hl,hl ; hl=(4-b)x128
 ld de,track1
 add hl,de
 ret


;-------------
; SHOW TRACKS
;-------------
; in: (vm) - view mode
; out: screen updated with all domino trains
;-------------------------------------------
showTracks:
 ld a,(vm)
 or a
 jr z,showTrack0
 jr showTrack2

;-------------
; VIEW MODE 0
;-------------
;   show last piece of each player and a train if they have one
;   need 32x8 block for each player (difficult for p2 and TTrain)
;    show last piece in first 16 pixels, then train in next 16
;    overwrite train with your selected domino
;---------------------------------------------
showTrack0:
 ld b,4
st0_loop:
 ; get track start
 ld a,4
 sub b
 call getTrackA

 push bc
  call showTrain0
 pop bc
 djnz st0_loop
 ret

showTrain0:
 ld a,(hl)
 ld (imCounter),a
 ; is it a virgin track?
 bit 4,a
 jr nz,st0_noPieces

 ; get location
 call st0_location
 
 ; get domino pieces
 push bc
  call endOfTrack ; hl=pointer to track
 pop bc
 ld (imCounter),a
 dec hl
 dec hl
 dec hl
 dec hl
 ld a,(hl)
 inc hl

 ; draw piece
 call drawPiece

 jr st0_drawTrain

; read from centerpiece rather than last domino on track
st0_noPieces:
 ; funky bit shifting to decide if piece is vert/horiz
 ;  shift byte *******x to x0000000
 ld a,b
 and 1b
 ld c,a
 sla c
 sla c
 sla c
 sla c
 sla c
 sla c
 sla c

 ; draw centerpiece
 call st0_location
 ld a,(centerpiece)
 ld hl,tempTrack+1
 ld (hl),a	; drawPiece pulls from HL not B
 or c
 call drawPiece

 ; draw a train if it exists (and does not block the cursor)
st0_drawTrain:
 ld a,(imCounter)
 ; is there a train?
 bit 5,a
 ret z

 ld a,4
 sub b

 call vm0_location
 push de
  ld a,13
  call drawDomino
 pop de
 ld a,8
 add a,d
 ld d,a
 ld a,14
 jp drawDomino 


; in: b
; out: (d,e) = (x,y) for sprite
st0_location:
 push bc
 push hl
  ld a,4
  sub b
  sla a
  ld e,a
  ld d,0
  ld hl,trainLocations
  add hl,de
  ld d,(hl)
  inc hl
  ld e,(hl)
 pop hl
 pop bc
 ret

; used by drawTrain
vm0_location:
 push hl
  sla a
  ld e,a
  ld d,0
  ld hl,trainLocations+8
  add hl,de
  ld d,(hl)
  inc hl
  ld e,(hl)
 pop hl
 ret

;-------------
; VIEW MODE 1
;-------------
;   eventually show the virtual screen...
;---------------------------------------------
showTrack1:
 ld hl,centerpiece
 call drawTrack
 ld a,0
 ld (imCounter),a
 ld hl,track1
 call drawTrack
 ld a,1
 ld (imCounter),a
 ld hl,track2
 call drawTrack
 ld a,2
 ld (imCounter),a
 ld hl,track3
 call drawTrack
 ld a,3
 ld (imCounter),a
 ld hl,track4
 call drawTrack
 ret

drawTrack:
 ld a,(hl)
 bit 5,a
 jr nz,drawTrainSprite
 bit 4,a
 ret nz

 call drawTrain
 jr drawTrack

; in: hl = pointer to train
drawTrain:
 inc hl
 inc hl
 ld d,(hl)
 inc hl
 ld e,(hl)
 dec hl
 dec hl

 call drawPiece

 inc hl
 inc hl
 inc hl
 ret

drawTrainSprite:
 bit 4,a
 jr z,notFirstTS

 ld hl,centerpiece+4
notFirstTS:
 dec hl
 ld e,(hl)
 dec hl
 ld d,(hl)

 ld a,(imCounter)
 or a
 jr z,dtsRight
 cp 1
 jr z,dtsDown
 cp 2
 jr z,dtsLeft

 ; up
 ld a,e
 sub 8
 ld e,a
 jr drawTS

dtsRight:
 ld a,16
 add a,d
 ld d,a
 jr drawTS

dtsDown:
 ld a,16
 add a,e
 ld e,a
 jr drawTS

dtsLeft:
 ld a,d
 sub 14
 ld d,a
drawTS:
 ld a,10
 push de
 push hl
  call drawDomino
 pop hl
 pop de
 ld a,8
 add a,d
 ld d,a
 ld a,11
 jp drawDomino

;--------------------------
; drawPiece
;--------------------------
;  draw a piece
; in: a (piece #),d,e (x,y coord), hl (pointer to second byte of domino)
; out: piece drawn to graph buffer
;--------------------------
drawPiece:
 push bc
 bit 7,a
 jr nz,drawPieceVertical

 and 1111b
 push de
 push hl
  call drawDomino
 pop hl
 pop de

 ld a,d
 add a,8
 ld d,a
 ld a,(hl)
 push hl
  call drawDomino
 pop hl
 pop bc
 ret

drawPieceVertical:
 and 1111b
 push de
 push hl
  call drawDomino
 pop hl
 pop de

 ld a,e
 add a,8
 ld e,a
 ld a,(hl)
 push hl
  call drawDomino
 pop hl
 pop bc
 ret



showTrack2:
 ld b,4
st2_loop:
 ; get track start
 ld a,4
 sub b
 call getTrackA

 push bc
  ld c,0
  call showTrain2
 pop bc
 djnz st2_loop
 ret

showTrain2:
 ld a,(hl)
 bit 4,a
 jr z,showTrain2go

 push hl
 push bc
  ld a,4
  sub b
  sla a
  sla a
  sla a
  ld (penrow),a
  ld a,20
  ld (pencol),a
  ld a,(hl)
  call showBits

  push hl
   ld a,Lperiod
   B_CALL vputmap
  pop hl

  inc hl
  inc hl
  inc hl
  inc hl
  ld a,(hl)
  call showBits
 pop bc
 pop hl
 ret

showTrain2go:
 push bc
 push hl
  push hl
   ld a,c
	add a,c
	add a,c
	add a,a
	ld h,a

	ld a,5
	sub b
	sla a
	sla a
	sla a
	sla a
	ld l,a
	ld (pencol),hl
  pop hl
  ld a,(hl)
  and 11111b
  call valueForA
  push hl
   ld a,Lperiod
   B_CALL vputmap
  pop hl
  inc hl
  ld a,(hl)
  call valueForA
 pop hl
 pop bc
 inc c
 inc hl
 inc hl
 inc hl
 inc hl
 jr showTrain2
  

